#!/usr/bin/env python
# -*- coding: utf-8 -*-
import random
import string
from poco.exceptions import PocoNoSuchNodeException
from poco.drivers.android.uiautomation import AndroidUiautomationPoco
poco = AndroidUiautomationPoco()
def random_Swipe(nodeName,childNode):
    node = nodeName
    item = node.child()
    if item.exists():
        count = len(list(item))
        print(count)
        print("***Ding!***")
        try:
            item[random.randint(0,count)].click()
            print("***Bingo!***")
        except PocoNoSuchNodeException:
            print('No item')
def random_Swipe1(nodeName):
    node = nodeName.child()
    #item = poco("node")
    if node.exists():
        count = len(list(node))
        print(count)
        print("***Ding!***")
        #node([random.randint(0,count)]).click()
        try:
            node[random.randint(0,count-1)].click()
            print("***Bingo!***")
        except PocoNoSuchNodeException:
            print('No item')


#/usr/bin/env python
# -*- coding: UTF-8 -*-

from poco.drivers.ios import iosPoco
poco = iosPoco()
import random
from poco.exceptions import *
from pocoui_lib.ios import kotoList

# 适用单选列表
def random_item():
    # 等待列表渲染完成
    table = poco("Table")
    kotoList.random_item(table)


def is_page(name):
    return poco(label=name).exists()



#/usr/bin/env python
# -*- coding: UTF-8 -*-

from poco.drivers.ios import iosPoco
poco = iosPoco()
import random
from poco.exceptions import *

# 适用单选列表
def random_item(tablePoco):
    # 等待列表渲染完成
    table = tablePoco
    table.wait(timeout=5)
    firstnode = poco("Table").child("Cell")[0]
    firstnode.wait(timeout=5)

    # 随机点击
    itemlist = poco("Table").child("Cell")
    # 可能会造成 Click position out of screen
    count = len(list(itemlist))

    if count > 2:
        end = count - 2
    else:
        end = count - 1
    index = random.randint(0, end)

    try:
        itemlist[index].click()
    except PocoNoSuchNodeException:
        print('list not exit item[%d]' % (index))

# 找到scrollview上的元素，滑倒底部元素
def scroll_to(nodeName, lastnodeName="保存"):
    n = poco(nodeName)
    while not n:
        try:
            poco("Table").scroll(direction='vertical', percent=0.6, duration=1.0)
        except PocoNoSuchNodeException:
            print(('scroll not exit %s' % (nodeName)).encode(encoding='utf-8'))

        # 滑倒最底部
        if poco(lastnodeName):
            break

    return n



from pocoui_lib.android import kotoList
from pocoui_lib.android import kotoPage
from pocoui_lib.android import kotoComponent
from pocoui_lib.android import kotoYyq
__all__ = ["kotoList", "kotoPage", "kotoComponent", "kotoYyq"]



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import random
import string
# 获取手机号码
def get_phone_number():
    pre_lst = ["130", "131", "132", "133", "134", "135", "136", "137", "138", "139", "147", "150", "151", "152", "153", "155", "156", "157", "158", "159", "186", "187",
"188"]
    return random.choice(pre_lst) + ''.join(random.sample(string.digits, 8))
# 添加微信
def get_weixin():
    return "".join(random.sample(string.ascii_letters, 8))
#
def get_num():
    return "".join(random.sample(string.digits,2))
if __name__ == "__main__":
    get_phone_number()



#/usr/bin/env python
# -*- coding: UTF-8 -*-
import yaml
from pocoui_lib.crawler.path import KOPath
from pocoui_lib.crawler.config import KOConfig
import sys
import codecs
sys.stdout = codecs.getwriter("utf-8")(sys.stdout.detach())

import logging
logging.basicConfig(filename='../log/'+__name__+'.log',format='[%(asctime)s-%(filename)s-%(levelname)s:%(message)s]', level = logging.DEBUG,filemode='w',datefmt='%Y-%m-%d %I:%M:%S %p')

class KOCrawler(object):
    """
    node检索，dfs，poco
    """
    def __init__(self, filepath):
        # 路径管理
        self.path = KOPath()
        # 配置信息管理
        KOConfig().loadDefault(filepath)

    def explore(self):
        """
        执行方法
        """
        # 登录操作
        # 到首页的操作，生成tabbar，如果页面里面有tabbar暂时不考虑了
        root = self.path.generate_root()
        self.path.advance_branch(root)
        self.dfs(root)

    def dfs(self, branch):
        """
        遍历branch
        """

        # 判断page是否检索完
        if branch.check_finished():
            print('[branch: %s] explore end \n' % (branch.identify))
            # 回到前一页继续检索
            previous_branch = self.path.back_to_previous_branch()

            if previous_branch:
                self.dfs(previous_branch)
            else:
                return
        for node in branch.node_list:
            # 找到没有检索的node继续检索
            if node not in branch.node_traversed_list:
                branch.node_traversed_list.append(node)
                branch.node_list.remove(node)
                node.trigger_action()
                print('[node: %s, pos:%s] action end \n' % (node.poco.attr('name'), node.poco.attr('pos')))
                self.dfs(self.path.explore_branch(node))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from poco.exceptions import PocoNoSuchNodeException
from poco.drivers.android.uiautomation import AndroidUiautomationPoco
poco = AndroidUiautomationPoco()
from pocoui_lib.android import kotoList
from pocoui_lib.android import kotoPage
import random
# 处理一般的pickerView
def picker_swipe():
    poco('android.widget.NumberPicker').wait(timeout=6)
    if poco('android.widget.NumberPicker').exists():
        poco('android.widget.NumberPicker').swipe('up')
        print(poco(text="完成").exists())
        if poco(text="完成").exists():
            poco(text="完成").click()
        else:
            print('完成 note existes'.encode(encoding='utf-8'))

# 选项数组随机点击，单选
def random_item_click(array):
    index = random.randint(0, len(array)-1)
    item = array[index]
    print('-------------random_item_click %d-----------------' % (index))
    # print("click the point", item)
    poco(text=item).wait(timeout=10)
    if(poco(text=item).exists()):
        poco(text=item).click(sleep_interval=6.0)
    else:
        print(('%s not exists' % (item)).encode(encoding='utf-8'))
        pass
    return item
# 点击之后延长秒
def polong_click(pocoPoint,time=6.0):
    # print("click the point", item)
    if(pocoPoint.exists()):
        pocoPoint.click(sleep_interval=time)
    else:
        print('point not exists')
        pass



#/usr/bin/env python
# -*- coding: UTF-8 -*-

import logging
from pocoui_lib.crawler.iosBranch import KOIOSBranch
from pocoui_lib.crawler.iosNode import KOIOSNode
from pocoui_lib.crawler.config import KOConfig
import time

class KOPath(object):
    """docstring for Path."""
    def __init__(self):
        # 当前操作的页面
        self.current_branch = None
        # 当前所有page的堆栈
        self._stack = []

    def generate_root(self):
        if KOConfig().config['platform'] == 'ios':
            branch = KOIOSBranch()
            branch.identify = "root"
            branch.find_root()
        return branch

    def check_nodes(self, branch):
        """
        通过node判断是否新建branch，并获取到新建的node, 如果在一个页面上的弹层，暂且看成是两个页面
        """
        if self.current_branch:
            # 如果原生页面的id是一样的
            if (self.current_branch.identify == branch.identify):
                # node的数量不同，就代表是新的页面
                if len(self.current_branch.node_list) != len(branch.node_list):

                    # tmp = []
                    # for m in branch.node_list:
                    #     for n in self.current_branch.node_list:
                    #         if not KOIOSNode.compare(m, n):
                    #             tmp.append(m)
                    branch.node_list = []
                    self.advance_branch(branch)
                    return branch
                # 数量相同代表是同一个页面
                else:
                    return self.current_branch
            else:
                self.advance_branch(branch)
                return branch
        else:
            self.advance_branch(branch)
            return branch


    def explore_branch(self, node=None):
        time.sleep(2)
        # TODO; 判断iOS还是android
        if KOConfig().config['platform'] == 'ios':
            branch = KOIOSBranch()
        branch.explore_nodes()
        branch = self.check_nodes(branch)
        branch.parent_poco = node.poco

        print("=============== branch %s nodes ===================" % (branch.identify))
        print(" || ".join(str(p.poco.attr('name')) for p in branch.node_list))
        print("================================================== \n")

        return branch

    def advance_branch(self, branch):
        """
        加入一个页面
        """
        self._stack.append(branch)
        self.current_branch = branch
        print('enter branch: %s' % (branch.identify))
        self._log_branch_path()

    def back_to_previous_branch(self):
        """
        回到上个页面
        """
        self.current_branch.back()
        # 延时处理
        time.sleep(2)
        self._stack.pop()
        branch = self._stack[-1]

        if (branch):
            self.current_branch = branch
            print('back to branch: %s' % (branch.identify))
            self._log_branch_path()
            return branch
        else:
            return None

    def _log_branch_path(self):
        print("=============== branch path ======================")
        print(" => ".join(str(b.identify) for b in self._stack))
        print("================================================== \n")



#/usr/bin/env python
# -*- coding: UTF-8 -*-
from poco.drivers.android.uiautomation import AndroidUiautomationPoco
poco = AndroidUiautomationPoco()

from pocoui_lib.android import kotoList


# 适用单选列表
def random_item():
    # 等待列表渲染完成
    kotoList.random_item()


def is_page(name):
    return poco(text=name).exists()



#/usr/bin/env python
# -*- coding: UTF-8 -*-
from functools import wraps
import yaml

def singleton(cls):
    instances = {}
    @wraps(cls)
    def getinstance(*args, **kw):
        if cls not in instances:
            instances[cls] = cls(*args, **kw)
        return instances[cls]
    return getinstance

@singleton
class KOConfig(object):
    """docstring for .KOConfig"""
    def __init__(self):
        self.config = None
        # branch 自增
        self.increment_num = 0
        # 首页tabbar数组
        self.tabbar_text = []

    def loadDefault(self, filepath):
        with open(filepath, 'r', encoding='utf8') as loadfile:
            self.config = yaml.load(loadfile)
            print(self.config)

    def increase(self):
        self.increment_num = self.increment_num + 1
        return self.increment_num



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import random
import string
import time
from poco.exceptions import PocoNoSuchNodeException
from poco.drivers.android.uiautomation import AndroidUiautomationPoco
poco = AndroidUiautomationPoco()
# recyclerview随机点击
def random_item():
    # 等待列表渲染完成
    table = poco(type='android.support.v7.widget.RecyclerView')
    table.wait(timeout=10)
    itemlist =table.child()
    firstnode = itemlist[0]
    firstnode.wait(timeout=10)
    # 可能会造成 Click position out of screen
    count = len(list(itemlist))
    if count > 2:
        end = count - 2
    else:
        end = count - 1
    index = random.randint(0, end)

    try:
        itemlist[index].click()
    except PocoNoSuchNodeException:
        print('list not exist item[%d]' % (index))
# 找到scrollview 并滑动到底部
def scroll_to(nodeName, percents, lastNodeName):
    scrollView = poco(type='ScrollView')
    if not scrollView.exists():
         scrollView = poco(type='android.widget.FrameLayout')
    n = poco(text=nodeName)
    lastNode = poco(text=lastNodeName)
    if scrollView.exists():
        while not n:
            try:
                if(percents>0):
                    scrollView.scroll(direction='vertical', percent=percents, duration=1.0)
                else:
                    scrollView.scroll(direction='vertical', percent=percents, duration=1.0)
            except PocoNoSuchNodeException:
                print('scroll not exist %s ' % (nodeName))
        # 滑倒最底部poco("com.souche.fengche:id/scrollView_custom")
            if lastNode.exists():
                break
        if not n.exists():
            print((' %s not exists' % (nodeName)).encode(encoding='utf-8'))
    else:
        print((' %s not exists' % (nodeName)).encode(encoding='utf-8'))
    time.sleep(5)
    return n
# recyclerview 传poco(id)
def random_poco_point_item(pocoPoint):
    # 等待列表渲染完成
    table = pocoPoint
    table.wait(timeout=10)
    itemlist =table.child()
    firstnode = itemlist[0]
    firstnode.wait(timeout=10)
    # 可能会造成 Click position out of screen
    count = len(list(itemlist))
    if count > 2:
        end = count - 2
    else:
        end = count - 1
    index = random.randint(0, end)

    try:
        itemlist[index].click()
    except PocoNoSuchNodeException:
        print('list not exist item[%d]' % (index))



#/usr/bin/env python
# -*- coding: UTF-8 -*-

from pocoui_lib.crawler.config import KOConfig

class KOBranch(object):
    """
    """
    def __init__(self):
        # 标记
        self.identify = str(KOConfig().increase())
        # 待检索node
        self.node_list = []
        # 完成检索的node
        self.node_traversed_list = []
        # 页面是否可以滑动
        self.can_scroll = False
        # 返回按钮
        self.back_poco = None
        # 某些branch是poco点击后出现的
        self.parent_poco = None
        # 页面类型
        self.branch_type = None

    def back(self):
        """
        点击返回按钮,文字和左上角位置
        """
        pass

    def explore_nodes(self):
        """
        根据一定的规则，遍历node
        """
        pass

    def check_finished(self):
        """
        通过列表和已点击列表对比是否完成检索
        """
        if len(self.node_list) == 0:
            return True
        return False
    def refresh(self):
        """
        页面刷新，重新添加node
        """
        pass



#/usr/bin/env python
# -*- coding: UTF-8 -*-

from poco.drivers.ios import iosPoco
poco = iosPoco()
import random
from poco.exceptions import *
from pocoui_lib.ios import kotoList
from pocoui_lib.ios import kotoPage

# 相册选择
def photo_picker():
    itemlist = poco("CollectionView").child("Cell")
    # 可能会造成 Click position out of screen
    count = len(list(itemlist))
    end = count - 1
    index = random.randint(0, end)

    try:
        itemlist[index].child("Button").click()
    except PocoNoSuchNodeException:
        print('list not exit item[%d]' % (index))

    if (poco("完成").exists()):
        poco("完成").click()

# 品牌车系二级联动，三级联动
def choose_brand():
    while kotoPage.is_page("选择品牌"):
        table = poco("Window").poco("Table")[0]
        kotoList.random_item(table)


# 处理一般的pickerView
def picker_swipe():
    if (poco("Picker").exists()):
        for wheel in poco("Picker").child("PickerWheel"):
            wheel.swipe('up')

    if (poco("完成").exists()):
        poco("完成").click()



# 选项数组随机点击，单选
def random_item_click(array):
    item = random.choice(array)
    poco(item).click()
    return item



#/usr/bin/env python
# -*- coding: UTF-8 -*-

from pocoui_lib.crawler.node import KONode
from enum import Enum
import time

class KOIOSNodeType(Enum):
    Application = 0
    Window = 1
    NavigationBar = 2
    TabBar = 3
    ScrollView = 4
    WebView = 5
    Table = 6
    Cell = 7
    StaticText = 8
    TextField = 9
    Button = 10
    Image = 11

class KOIOSNodeAction(Enum):
    click = 1

class KOIOSNode(KONode):
    """docstring for Page."""

    @classmethod
    def compare(cls, node, anothernode):
        if (node.poco.attr('name') == anothernode.poco.attr('name')):
        # and (node.poco.attr('pos') == anothernode.poco.attr('pos')):
            return True
        else:
            return False

    def trigger_action(self):
        time.sleep(2)
        if self.poco.exists:
            if self.action == KOIOSNodeAction.click:
                self.poco.click()



#/usr/bin/env python
# -*- coding: UTF-8 -*-
from poco.exceptions import PocoNoSuchNodeException
from poco.drivers.android.uiautomation import AndroidUiautomationPoco
poco = AndroidUiautomationPoco()
from pocoui_lib.android import kotoList
from pocoui_lib.android import kotoPage
import random


def swipe_find(poco1,precent):
    #找不到元素像下滑动屏幕
    while poco1.exists()==False:
        print("not exist...")
        poco().swipe(precent)
    else:
        print("exist...")
        return poco1

def multi_select_poco_point_item(pocoPoint,count):
    #列表多选,入参count为多选数量
    table = pocoPoint
    table.wait(timeout=10)
    itemlist =table.child()
    counts = len(list(itemlist))
    a = random.sample(range(0,counts),count)    #选择0到counts-1中，不重复数count次
    for i in a:
        itemlist[i].click()
        pocoPoint.swipe([0,-0.5])
        j = i+1
        print("select the %s " %j)


def all_poco_point_item(pocoPoint):
    # 选择列表所有选项
    table = pocoPoint
    table.wait(timeout=10)
    itemlist =table.child()
    # 可能会造成 Click position out of screen
    count = len(list(itemlist))
    print("=================success")
    print(count)
    print("=================success")
    for i in range(0,count):
    	itemlist[i].click()
    # try:
    #     itemlist[index].click()
    # except PocoNoSuchNodeException:
    #     print('list not exist item[%d]' % (index))
# #时间控件，滚动年、月
# def time(pocoy):
# 	for i in range(3,8):
# 		pocoy.swipe([0.1,-0.1])
# 	print("=================success")



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages

setup(name='pocoui_lib',
      version='1.1.2',
      description='pocoui lib for souche-inc airtest project',
      url='',
      author='mizhdi',
      author_email='mizhdi@gmail.com',
      license='MIT',
      platforms = "any",
      install_requires = ["pocoui", "pyyaml"],
      packages = find_packages(),
      zip_safe=False)



#/usr/bin/env python
# -*- coding: UTF-8 -*-
from pocoui_lib.crawler.branch import KOBranch
from pocoui_lib.crawler.iosNode import KOIOSNode
from pocoui_lib.crawler.iosNode import KOIOSNodeType
from pocoui_lib.crawler.iosNode import KOIOSNodeAction
from poco.drivers.ios import iosPoco
import logging
import json
import re
poco = iosPoco()
from enum import Enum

class KOIOSBranchType(Enum):
    ReactNative = 1,
    Native = 2,
    WebView = 3,
    NavPage = 4

class KOIOSBranch(KOBranch):
    cell_max_count = 4
    action_node_array = ['Button'];
    scroll_node_array = ['WebView', 'ScrollView', 'Table']

    """
    ios的branch页管理
    """
    def __init__(self):
        KOBranch.__init__(self)
        self.cell_num = 0
        # 事件点击后的回调
        poco.add_post_action_callback(self._action_callback)

        # 默认是rn
        self.branch_type = KOIOSBranchType.ReactNative

    def back(self):
        if self.back_poco:
            if self.back_poco.exists():
                self.back_poco.click()
        elif self.branch_type == KOIOSBranchType.ReactNative:
            poco.click([0.08, 0.06])
        elif self.parent_poco:
            if self.parent_poco.exists():
                self.parent_poco.click()
        else:
            # 如果是弹层，点击位置像是点击两次
            poco.click([0.08, 0.06])

    def check_back_button(self, ui):
        # pos = ui.attr('pos')
        name = ui.attr('name')
        # pattern = re.compile(r"取消|返回|back")
        if 'back' in name:
            self.back_poco = ui
            return True
        else:
            return False

    def explore_nodes(self):
        self.node_list = self.refresh()
        return self.node_list

    def _action_callback(self, poco, action, ui, args):
        # print('[name: %s, type: %s, action: %s]' % (str(ui.attr('name').encode('utf-8')), ui.attr('type'), action))
        #
        # 找到新增的元素，赋值parentpoco
        pass

    def find_root(self):
        # 如果是tabbar要特殊处理
        if poco(type="TabBar"):
            for button in (poco(type="TabBar").child(type="Button")):
                self.node_list.append(KOIOSNode(button))
        return self.node_list

    def refresh(self):
        array_tmp = []
        # print(*poco(), sep='\n')

        for ui in poco():
            # dump出整个UI树
            # print(json.dumps(poco.agent.hierarchy.dump(), indent=2))
            # branch_dict = poco.agent.hierarchy.dumper.dumpHierarchyImpl(root.nodes[0], True)
            node_type = ui.attr('type')
            type = KOIOSNodeType.StaticText
            action = KOIOSNodeAction.click
            # 找到页面id
            if node_type == 'NavigationBar':
                type = KOIOSNodeType.NavigationBar
                if ui.attr('identifier'):
                    self.identify = str(ui.attr('identifier'))
                else:
                    self.identify = str(ui.attr('name'))
                self.branch_type = KOIOSBranchType.Native

            if node_type == 'WebView':
                type = KOIOSNodeType.WebView
                self.branch_type = KOIOSBranchType.WebView

            if node_type in KOIOSBranch.scroll_node_array:
                self.can_scroll = True
            # 将可以点击的点加入到数组中
            if node_type in KOIOSBranch.action_node_array:

                if node_type == 'Button':
                    type = KOIOSNodeType.Button
                    if self.check_back_button(ui):
                        continue
                    # TODO: 排除tabbar的node
                if node_type == 'Cell':
                    self.cell_num = self.cell_num + 1
                    if self.cell_num == KOIOSBranch.cell_max_count:
                        continue
                node = KOIOSNode(ui)
                node.type = type
                node.action = action
                array_tmp.append(node)

        # TODO: 查看当前页面是否有TabBar
        if poco(type="TabBar"):
            for button in poco(type="TabBar").child(type="Button"):
                m = KOIOSNode(button)
                for n in array_tmp:
                    if KOIOSNode.compare(m, n):
                        array_tmp.remove(n)

        return array_tmp



#/usr/bin/env python
# -*- coding: UTF-8 -*-

class KONode(object):
    """docstring for node."""
    def __init__(self, poco, parentPoco=None):
        # 当前的poco
        self.poco = poco
        # poco的操作
        self.action = None
        # 当前node的类型
        self.type = None

        self.parent_branch = None

    def trigger_action(self):
        pass
